{
  Programm : SWITCH.PAS
  Sprache  : Delphi 1.0
  Zweck    : Schalter-Komponente
  Datum    : 15, 16. Feb. 1996
  Update   : April 1999
  Autor    : U.Juerss

  Component "Switch V2.0" simulates an eletric toggle switch without using bitmaps.

  (Part of LMD-tools, programmed by me in 1996, updated 1999)

  Udo Juerss
  57078 Siegen, Germany
  April 1999

  e-mail: ujhs@aol.com
}

unit
  Switch;

interface

uses
  Windows,Messages,Classes,Controls,Graphics;
{------------------------------------------------------------------------------}

type
  RectArray = array[0..3] of TPoint;               {Vektorarraytyp fr Rechteck}
  TriArray = array[0..2] of TPoint;                 {Vektorarraytyp fr Dreieck}

  TSwitch = class(TCustomControl)
  private
    TopShape: TriArray;                 {Dreieck Vektoren von Schalteroberseite}
    OnShape: RectArray;               {Rechteck Vektoren von Schalterfront "ON"}
    OffShape: RectArray;             {Rechteck Vektoren von Schalterfront "OFF"}
    SideShape: RectArray;                  {Rechteck Vektoren von Schalterseite}

    FOnChanged: TNotifyEvent;                        {Verbindung zur Aussenwelt}
    FOnChecked: TNotifyEvent;                        {Verbindung zur Aussenwelt}
    FOnUnChecked: TNotifyEvent;                      {Verbindung zur Aussenwelt}

    FCaptionOn: TCaption;                   {Beschriftung Schalterstellung "ON"}
    FCaptionOff: TCaption;                 {Beschriftung Schalterstellung "OFF"}
    FChecked: Boolean;                               {Flag von Schalterstellung}
    FCheckedLeft: Boolean;     {Flag ob "ON" links oder rechts dargestellt wird}
    FSlope: Byte;                            {Neigung (3D Effekt) des Schalters}
    FSideLength: Byte;          {Seitenabstand fr hervorstehendes Schalterteil}
    FOnColor: TColor;                               {Farbe fr Frontflche "ON"}
    FOffColor: TColor;                             {Farbe fr Frontflche "OFF"}
    FTopColor: TColor;                             {Farbe fr Schalteroberseite}
    FSideColor: TColor;                                 {Farbe fr Seitenflche}
    ALeft: Integer;                        {Linke Anfangsposition des Schalters}
    ATop: Integer;                         {Obere Anfangsposition des Schalters}
    AHeight: Integer;                                       {Hhe des Schalters}
    AWidth: Integer;                                      {Breite des Schalters}
    LabelLen: Integer;                                {Halbbreite des Schalters}
    LabelOfs: Integer;                       {Halbbreite fr Spiegeldarstellung}
    Side: Integer;                                 {Temporr in Setup verwendet}

    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure CallNotifyEvent;
    procedure Setup;
    procedure Draw;
    procedure SetCaption(Index: Integer; Value: TCaption);
    procedure SetBoolean(Index: Integer; Value: Boolean);
    procedure SetByte(Index: Integer; Value: Byte);
    procedure SetColor(Index: Integer; Value: TColor);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  published
    property Align;
    property Anchors;
    property Font;
    property TabStop;
    property TabOrder;
    property ShowHint;
    property CaptionOn: TCaption index 0 read FCaptionOn write SetCaption;
    property CaptionOff: TCaption index 1 read FCaptionOff write SetCaption;
    property Checked: Boolean index 0 read FChecked write SetBoolean;
    property CheckedLeft: Boolean index 1 read FCheckedLeft write SetBoolean;
    property Slope: Byte index 0 read FSlope write SetByte;
    property SideLength: Byte index 1 read FSideLength write SetByte;
    property OnColor: TColor index 0 read FOnColor write SetColor;
    property OffColor: TColor index 1 read FOffColor write SetColor;
    property TopColor: TColor index 2 read FTopColor write SetColor;
    property SideColor: TColor index 3 read FSideColor write SetColor;

    property OnClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnKeyDown;
    property OnKeyPress;
    property OnChanged: TNotifyEvent read FOnChanged write FOnChanged;
    property OnChecked: TNotifyEvent read FOnChecked write FOnChecked;
    property OnUnChecked: TNotifyEvent read FOnUnChecked write FOnUnChecked;
  end;
{------------------------------------------------------------------------------}

procedure Register;

implementation
{------------------------------------------------------------------------------}

constructor TSwitch.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Caption:='';
  FCaptionOn:='ON';
  FCaptionOff:='OFF';
  FSlope:=6;
  FSideLength:=6;
  FChecked:=True;
  FCheckedLeft:=False;
  FOnColor:=clRed;
  FOffColor:=clMaroon;
  FTopColor:=clSilver;
  FSideColor:=clSilver;
  FOnChecked:=nil;
  FOnUnChecked:=nil;
  SetBounds(Left,Top,60,14 + FSlope);
  Font.Name:='small fonts';
  Font.Size:=7;
  Font.Color:=clWhite;
  TabStop:=True;
end;
{------------------------------------------------------------------------------}

procedure TSwitch.Paint;
begin
  Draw;            {Keine geerbte Methode aufrufen und sofort Schalter zeichnen}
end;
{------------------------------------------------------------------------------}

procedure TSwitch.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button,Shift,X,Y);
  if (Button = mbLeft) then
  begin
    SetFocus;
    if ((LabelLen > 0) and (X > LabelLen)) or
       ((LabelLen < 0) and (X < Abs(LabelLen))) then
    begin    {Nur wenn Mausklick innerhalb des hervorgehobenen Schalterteil ist}
      FChecked:=not FChecked;
      CallNotifyEvent;
      Invalidate;
    end;
  end;
end;
{------------------------------------------------------------------------------}

procedure TSwitch.WMSetFocus(var Message: TWMSetFocus);
begin
  Invalidate;
end;
{------------------------------------------------------------------------------}

procedure TSwitch.WMKillFocus(var Message: TWMKillFocus);
begin
  Invalidate;
end;
{------------------------------------------------------------------------------}

procedure TSwitch.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if Focused and ((Key = VK_Space) or (Key = VK_Return)) then
  begin
    FChecked:=not FChecked;
    CallNotifyEvent;
    Invalidate;
    Click;
  end;
end;
{------------------------------------------------------------------------------}

procedure TSwitch.CallNotifyEvent;                       {Auenwelt informieren}
begin
  if Assigned(FOnChanged) then FOnChanged(Self);
  if FChecked and Assigned(FOnChecked) then FOnChecked(Self) else
  if not FChecked and Assigned(FOnUnChecked) then FOnUnChecked(Self);
end;
{------------------------------------------------------------------------------}

procedure TSwitch.Draw;                                      {Schalter zeichnen}
var
  TW: Integer;
  TH: Integer;
begin
  Setup;                                  {Vektoren fr Schalterteile berechnen}
  if Focused then Canvas.Rectangle(0,0,Width,AHeight + 1 + 2 * ATop);
  Canvas.Pen.Color:=clWhite;                   {Umrandung von Schalter zeichnen}
  Canvas.MoveTo(ALeft - 1,ATop + AHeight + 1);
  Canvas.LineTo(ALeft + AWidth,ATop + AHeight + 1);      {Untere Linie in weiss}
  Canvas.LineTo(ALeft + AWidth,ATop - 2);                {Rechte Linie in weiss}

  Canvas.Pen.Color:=clGray;
  Canvas.MoveTo(ALeft + AWidth,ATop - 1);
  Canvas.LineTo(ALeft - 1,ATop - 1);                 {Obere Linie in dunkelgrau}
  Canvas.LineTo(ALeft - 1,ATop + AHeight + 1);       {Linke Linie in dunkelgrau}

  Canvas.Pen.Color:=clBlack;                      {Polygonumrandung ist schwarz}
  Canvas.Brush.Style:=bsSolid;                      {Fllflche ist geschlossen}
  Setup;
  Canvas.Brush.Color:=FTopColor;
  Canvas.Polygon(TopShape);                         {Top des Schalters zeichnen}
  Canvas.Brush.Color:=FSideColor;
  Canvas.Polygon(SideShape);                      {Seite des Schalters zeichnen}
  if FChecked then Canvas.Brush.Color:=FOnColor
  else Canvas.Brush.Color:=FOffColor;
  Canvas.Polygon(OnShape);                     {On Seite des Schalters zeichnen}
  Canvas.Brush.Color:=FOffColor;
  Canvas.Polygon(OffShape);                   {Off Seite des Schalters zeichnen}

  Canvas.Font:=Font;                                  {Gewhlten Font bergeben}
  Canvas.Brush.Style:=bsClear;                        {Transparente Textausgabe}

  if FChecked then Caption:=FCaptionOn else Caption:=FCaptionOff;

  if LabelLen > 0 then TW:=ALeft + ((Abs(LabelLen) - Canvas.TextWidth(Caption)) div 2)
  else TW:=LabelOfs + ((Abs(LabelLen) - Canvas.TextWidth(Caption)) div 2);
  TH:=ATop + ((AHeight - Canvas.TextHeight(Caption)) div 2);

  Canvas.TextOut(TW,TH,Caption);
end;
{------------------------------------------------------------------------------}

procedure TSwitch.Setup;                  {Vektoren fr Schalterteile berechnen}
begin
  ALeft:=2;                {2 Pixel linker Abstand fr Rahmen und Focusrechteck}
  ATop:=2;                 {2 Pixel oberer Abstand fr Rahmen und Focusrechteck}
  AHeight:=Height - FSlope - 2 * ATop;   {Schalterhhe = Height - Ofs - Neigung}
  AWidth:=Width - 2 * ALeft;                  {Schalterbreite = Width - 2 * Ofs}
  LabelLen:=AWidth div 2;
  LabelOfs:=LabelLen + ALeft;
  Side:=FSideLength;
  if (not FChecked and FCheckedLeft) or (not FCheckedLeft and FChecked) then
  begin
    LabelLen:=-LabelLen;
    Side:=-FSideLength;
  end;
  TopShape[0].X:=LabelOfs;          {Vektoren von obere Dreieckflche berechnen}
  TopShape[0].Y:=ATop;
  TopShape[1].X:=LabelOfs + LabelLen - Side;
  TopShape[1].Y:=ATop + FSlope;
  TopShape[2].X:=LabelOfs + LabelLen;
  TopShape[2].Y:=ATop;

  OnShape[0].X:=LabelOfs - LabelLen;   {Vektoren der "EIN" Frontseite berechnen}
  OnShape[0].Y:=ATop;
  OnShape[1]:=TopShape[0];
  OnShape[2]:=OffShape[3];
  OnShape[3].X:=OnShape[0].X;
  OnShape[3].Y:=ATop + AHeight;

  OffShape[0]:=TopShape[0];            {Vektoren der "AUS" Frontseite berechnen}
  OffShape[1]:=TopShape[1];
  OffShape[2].X:=OffShape[1].X;
  OffShape[2].Y:=OffShape[1].Y + AHeight;
  OffShape[3].X:=OffShape[0].X;
  OffShape[3].Y:=ATop + AHeight;

  SideShape[0]:=OffShape[1];               {Vektoren der Seitenflche berechnen}
  SideShape[1]:=TopShape[2];
  SideShape[2].X:=SideShape[1].X;
  SideShape[2].Y:=ATop + AHeight;
  SideShape[3]:=OffShape[2];
end;
{------------------------------------------------------------------------------}

procedure TSwitch.SetCaption(Index: Integer; Value: TCaption);
begin
  case Index of
    0: if FCaptionOn <> Value then
       begin
         FCaptionOn:=Value;
         Invalidate;
       end;
    1: if FCaptionOff <> Value then
       begin
         FCaptionOff:=Value;
         Invalidate;
       end;
  end;
end;
{------------------------------------------------------------------------------}

procedure TSwitch.SetBoolean(Index: Integer; Value: Boolean);
begin
  case Index of
    0: if FChecked <> Value then
       begin
         FChecked:=Value;
         CallNotifyEvent;
         Invalidate;
       end;
    1: if FCheckedLeft <> Value then
       begin
         FCheckedLeft:=Value;
         Invalidate;
       end;
  end;
end;
{------------------------------------------------------------------------------}

procedure TSwitch.SetByte(Index: Integer; Value: Byte);
begin
  case Index of
    0: if FSlope <> Value then
       begin
         FSlope:=Value;
         Invalidate;
       end;

    1: if (FSideLength <> Value) and (Value < Width - 4) then
       begin
         FSideLength:=Value;
         Invalidate;
       end;
  end;
end;
{------------------------------------------------------------------------------}

procedure TSwitch.SetColor(Index: Integer; Value: TColor);
begin
  case Index of
    0: if FOnColor <> Value then
       begin
         FOnColor:=Value;
         Invalidate;
       end;
    1: if FOffColor <> Value then
       begin
         FOffColor:=Value;
         Invalidate;
       end;
    2: if FTopColor <> Value then
       begin
         FTopColor:=Value;
         Invalidate;
       end;
    3: if FSideColor <> Value then
       begin
         FSideColor:=Value;
         Invalidate;
       end;
  end;
end;
{------------------------------------------------------------------------------}

procedure Register;
begin
  RegisterComponents('Udo',[TSwitch]);
end;
{------------------------------------------------------------------------------}

initialization
end.

